@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2009-2010 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2021 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2022 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,9 +36,11 @@
 
 #include "config.h"
 
+#include <errno.h>
 #include <gtk/gtk.h>
 #include <string.h>
 
+#include "remmina_log.h"
 #include "remmina_public.h"
 #include "remmina_pref.h"
 #include "remmina_string_array.h"
@@ -49,13 +51,17 @@
 static gchar *remminadir;
 static gchar *cachedir;
 
-/* return first found data dir as per XDG specs.
- * The returned string must be freed by the caller with g_free */
+/**
+ * Return datadir_path from pref or first found data dir as per XDG specs.
+ *
+ * The returned string must be freed by the caller with g_free
+ */
 gchar *remmina_file_get_datadir(void)
 {
 	TRACE_CALL(__func__);
 	const gchar *dir = ".remmina";
 	int i;
+
 	/* From preferences, datadir_path */
 	remminadir = remmina_pref_get_value("datadir_path");
 	if (remminadir != NULL && strlen(remminadir) > 0)
@@ -74,6 +80,7 @@ gchar *remmina_file_get_datadir(void)
 	g_free(remminadir), remminadir = NULL;
 	/* /usr/local/share/remmina */
 	const gchar *const *dirs = g_get_system_data_dirs();
+
 	g_free(remminadir), remminadir = NULL;
 	for (i = 0; dirs[i] != NULL; ++i) {
 		remminadir = g_build_path("/", dirs[i], "remmina", NULL);
@@ -99,6 +106,13 @@ static gboolean remmina_file_manager_do_copy(const char *src_path, const char *d
 	return ok;
 }
 
+/**
+ * It creates the Remmina data and cache folders
+ *
+ * If it finds the legacy ~/.remmina folder it copies the connection profiles into the new folder.
+ *
+ * If it finds default profiles in the XDG_DATA_DIRS it copies the profiles into the user data-folder.
+ */
 void remmina_file_manager_init(void)
 {
 	TRACE_CALL(__func__);
@@ -107,10 +121,21 @@ void remmina_file_manager_init(void)
 	const gchar *filename;
 	int i;
 
-	remminadir = g_build_path("/", g_get_user_data_dir(), "remmina", NULL);
-	/* Create the XDG_USER_DATA directory */
-	g_mkdir_with_parents(remminadir, 0750);
-	g_free(remminadir), remminadir = NULL;
+	/* Get and create the XDG_DATA_HOME directory */
+	remminadir = remmina_pref_get_value("datadir_path");
+	if (g_mkdir_with_parents(remminadir, 0750) == 0) {
+		REMMINA_DEBUG("Initialized the \"%s\" data folder", remminadir);
+		g_free(remminadir), remminadir = NULL;
+	} else {
+		g_free(remminadir), remminadir = NULL;
+		/* Get and create the XDG_DATA_HOME directory */
+		remminadir = g_build_path("/", g_get_user_data_dir(), "remmina", NULL);
+		if (g_mkdir_with_parents(remminadir, 0750) == 0)
+			REMMINA_DEBUG("Initialized the \"%s\" data folder", remminadir);
+		else
+			REMMINA_CRITICAL("Cannot create the \"%s\" data folder", remminadir);
+		g_free(remminadir), remminadir = NULL;
+	}
 	/* Create the XDG_CACHE_HOME directory */
 	cachedir = g_build_path("/", g_get_user_cache_dir(), "remmina", NULL);
 	g_mkdir_with_parents(cachedir, 0750);
@@ -129,6 +154,7 @@ void remmina_file_manager_init(void)
 
 	/* XDG_DATA_DIRS, i.e. /usr/local/share/remmina */
 	const gchar *const *dirs = g_get_system_data_dirs();
+
 	g_free(remminadir), remminadir = NULL;
 	for (i = 0; dirs[i] != NULL; ++i) {
 		remminadir = g_build_path("/", dirs[i], "remmina", NULL);
