@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2009-2011 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2021 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2022 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -200,10 +200,13 @@ static void remmina_ftp_client_cell_data_filetype_pixbuf(GtkTreeViewColumn *col,
 
 	switch (type) {
 	case REMMINA_FTP_FILE_TYPE_DIR:
-		g_object_set(renderer, "stock-id", "folder", NULL);
+		g_object_set(renderer, "icon-name", "folder", NULL);
+		break;
+	case REMMINA_FTP_FILE_TYPE_LINK:
+		g_object_set(renderer, "icon-name", "emblem-symbolic-link", NULL);
 		break;
 	default:
-		g_object_set(renderer, "stock-id", "text-x-generic", NULL);
+		g_object_set(renderer, "icon-name", "text-x-generic", NULL);
 		break;
 	}
 }
@@ -219,17 +222,17 @@ static void remmina_ftp_client_cell_data_progress_pixbuf(GtkTreeViewColumn *col,
 
 	switch (status) {
 	case REMMINA_FTP_TASK_STATUS_WAIT:
-		g_object_set(renderer, "stock-id", "P_ause", NULL);
+		g_object_set(renderer, "icon-name", "media-playback-pause", NULL);
 		break;
 	case REMMINA_FTP_TASK_STATUS_RUN:
-		g_object_set(renderer, "stock-id",
+		g_object_set(renderer, "icon-name",
 			(tasktype == REMMINA_FTP_TASK_TYPE_UPLOAD ? "go-up" : "go-down"), NULL);
 		break;
 	case REMMINA_FTP_TASK_STATUS_FINISH:
-		g_object_set(renderer, "stock-id", "_Yes", NULL);
+		g_object_set(renderer, "icon-name", "emblem-default", NULL);
 		break;
 	case REMMINA_FTP_TASK_STATUS_ERROR:
-		g_object_set(renderer, "stock-id", "_No", NULL);
+		g_object_set(renderer, "icon-name", "emblem-unreadable", NULL);
 		break;
 	}
 }
@@ -648,14 +651,14 @@ static void remmina_ftp_client_popup_menu(RemminaFTPClient *client, GdkEventButt
 
 	menuitem = gtk_menu_item_new_with_label(_("Download"));
 	gtk_widget_show(menuitem);
-	image = gtk_image_new_from_icon_name("remmina-document-save-symbolic", GTK_ICON_SIZE_MENU);
+	image = gtk_image_new_from_icon_name("org.remmina.Remmina-document-save-symbolic", GTK_ICON_SIZE_MENU);
 	gtk_widget_show(image);
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 	g_signal_connect(G_OBJECT(menuitem), "activate", G_CALLBACK(remmina_ftp_client_action_download), client);
 
 	menuitem = gtk_menu_item_new_with_label(_("Upload"));
 	gtk_widget_show(menuitem);
-	image = gtk_image_new_from_icon_name("remmina-document-send-symbolic", GTK_ICON_SIZE_MENU);
+	image = gtk_image_new_from_icon_name("org.remmina.Remmina-document-send-symbolic", GTK_ICON_SIZE_MENU);
 	gtk_widget_show(image);
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 	g_signal_connect(G_OBJECT(menuitem), "activate", G_CALLBACK(remmina_ftp_client_action_upload), client);
@@ -695,6 +698,9 @@ static gboolean remmina_ftp_client_file_list_on_button_press(GtkWidget *widget,
 			case REMMINA_FTP_FILE_TYPE_DIR:
 				remmina_ftp_client_open_dir(client, name);
 				break;
+			case REMMINA_FTP_FILE_TYPE_LINK:
+				remmina_ftp_client_open_dir(client, name);
+				break;
 			case REMMINA_FTP_FILE_TYPE_FILE:
 			default:
 				localdir = remmina_ftp_client_get_download_dir(client);
@@ -739,63 +745,46 @@ static GtkWidget* remmina_ftp_client_create_toolbar(RemminaFTPClient *client)
 	TRACE_CALL(__func__);
 	GtkWidget *box;
 	GtkWidget *button;
-	GtkWidget *image;
 	gint i = 0;
 
 	box = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
-	gtk_widget_show(box);
 	gtk_button_box_set_layout(GTK_BUTTON_BOX(box), GTK_BUTTONBOX_START);
 	gtk_grid_attach(GTK_GRID(client), box, 0, 0, 1, 1);
 
 	button = gtk_button_new_with_label(_("Home"));
-	gtk_widget_show(button);
 	gtk_widget_set_tooltip_text(button, _("Go to home folder"));
-	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
 	gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(remmina_ftp_client_action_home), client);
 
 	button = gtk_button_new_with_label(_("Up"));
-	gtk_widget_show(button);
 	gtk_widget_set_tooltip_text(button, _("Go to parent folder"));
-	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
 	gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(remmina_ftp_client_action_parent), client);
 
 	button = gtk_button_new_with_label(_("Refresh"));
-	gtk_widget_show(button);
 	gtk_widget_set_tooltip_text(button, _("Refresh current folder"));
-	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
 	gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(remmina_ftp_client_action_refresh), client);
 
 	button = gtk_button_new_with_label(_("Download"));
-	gtk_widget_show(button);
 	gtk_widget_set_tooltip_text(button, _("Download from server"));
-	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
-	image = gtk_image_new_from_icon_name("remmina-document-save-symbolic", GTK_ICON_SIZE_BUTTON);
-	gtk_button_set_image(GTK_BUTTON(button), image);
 	gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(remmina_ftp_client_action_download), client);
 
 	client->priv->file_action_widgets[i++] = button;
 
 	button = gtk_button_new_with_label(_("Upload"));
-	gtk_widget_show(button);
 	gtk_widget_set_tooltip_text(button, _("Upload to server"));
-	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
-	image = gtk_image_new_from_icon_name("remmina-document-send-symbolic", GTK_ICON_SIZE_BUTTON);
-	gtk_button_set_image(GTK_BUTTON(button), image);
 	gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(remmina_ftp_client_action_upload), client);
 
 	button = gtk_button_new_with_label(_("Delete"));
-	gtk_widget_show(button);
 	gtk_widget_set_tooltip_text(button, _("Delete files on server"));
-	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
 	gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(remmina_ftp_client_action_delete), client);
 
 	client->priv->file_action_widgets[i++] = button;
+	gtk_widget_show_all(box);
 
 	return box;
 }
@@ -879,6 +868,7 @@ static void remmina_ftp_client_init(RemminaFTPClient *client)
 
 	/* Toolbar */
 	toolbar = remmina_ftp_client_create_toolbar(client);
+	gtk_widget_set_hexpand(toolbar, TRUE);
 
 	/* The Paned to separate File List and Task List */
 	vpaned = gtk_paned_new(GTK_ORIENTATION_VERTICAL);
@@ -1031,7 +1021,7 @@ static void remmina_ftp_client_init(RemminaFTPClient *client)
 
 	renderer = remmina_cell_renderer_pixbuf_new();
 	column = gtk_tree_view_column_new_with_attributes(NULL, renderer, NULL);
-	g_object_set(G_OBJECT(renderer), "stock-id", "_Cancel", NULL);
+	g_object_set(G_OBJECT(renderer), "icon-name", "process-stop", NULL);
 	gtk_tree_view_column_set_resizable(column, FALSE);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(priv->task_list_view), column);
 
@@ -1203,7 +1193,11 @@ remmina_ftp_client_get_waiting_task(RemminaFTPClient *client)
 			path = gtk_tree_model_get_path(priv->task_list_model, &iter);
 			task.rowref = gtk_tree_row_reference_new(priv->task_list_model, path);
 			gtk_tree_path_free(path);
+#if GLIB_CHECK_VERSION(2,68,0)
+			return (RemminaFTPTask*)g_memdup2(&task, sizeof(RemminaFTPTask));
+#else
 			return (RemminaFTPTask*)g_memdup(&task, sizeof(RemminaFTPTask));
+#endif
 		}
 		if (!gtk_tree_model_iter_next(priv->task_list_model, &iter))
 			break;
