@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2009-2010 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2021 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2022 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -66,7 +66,10 @@ GType remmina_ftp_client_get_type(void)
 G_GNUC_CONST;
 
 enum {
-	REMMINA_FTP_FILE_TYPE_DIR, REMMINA_FTP_FILE_TYPE_FILE, REMMINA_FTP_FILE_N_TYPES,
+	REMMINA_FTP_FILE_TYPE_DIR,
+	REMMINA_FTP_FILE_TYPE_FILE,
+	REMMINA_FTP_FILE_TYPE_LINK,
+	REMMINA_FTP_FILE_N_TYPES,
 };
 
 enum {
