@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2009-2011 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2020 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2022 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,11 +34,13 @@
  *
  */
 
-#include "remmina_string_array.h"
-
 #pragma once
 
 #include "remmina_file.h"
+#include "remmina_monitor.h"
+#include <gtk/gtk.h>
+
+#include "remmina_string_array.h"
 
 typedef struct _RemminaMainPriv RemminaMainPriv;
 
@@ -57,6 +59,7 @@ typedef struct _RemminaMain {
 	GtkButton *		button_make_default;
 	/* Search bar objects */
 	GtkToggleButton *	search_toggle;
+	GtkSwitch *		switch_dark_mode;
 	GtkToggleButton *	view_toggle_button;
 	GtkToggleButton *	ustats_toggle;
 	GtkSearchBar *		search_bar;
@@ -67,15 +70,17 @@ typedef struct _RemminaMain {
 	GtkButton *		button_quick_connect;
 	/* Other widgets */
 	GtkTreeView *		tree_files_list;
+	GtkTreeViewColumn *	column_files_list_name;
 	GtkTreeViewColumn *	column_files_list_group;
+	GtkTreeViewColumn *	column_files_list_server;
+	GtkTreeViewColumn *	column_files_list_plugin;
+	GtkTreeViewColumn *	column_files_list_date;
 	GtkStatusbar *		statusbar_main;
+	GtkWidget *		network_icon;
 	/* Non widget objects */
 	GtkAccelGroup *		accelgroup_shortcuts;
-	/* Actions from the view ActionGroup */
-	GtkToggleAction *	action_view_quick_connect;
-	GtkToggleAction *	action_view_mode_list;
-	GtkToggleAction *	action_view_mode_tree;
 	RemminaMainPriv *	priv;
+	RemminaMonitor *	monitor;
 } RemminaMain;
 
 struct _RemminaMainPriv {
@@ -110,6 +115,7 @@ void remmina_main_on_action_application_news(GSimpleAction *action, GVariant *pa
 void remmina_main_on_action_application_default(GSimpleAction *action, GVariant *param, gpointer data);
 void remmina_main_on_action_application_mpchange(GSimpleAction *action, GVariant *param, gpointer data);
 void remmina_main_on_action_application_plugins(GSimpleAction *action, GVariant *param, gpointer data);
+void remmina_main_on_action_application_dark_theme(GSimpleAction *action, GVariant *param, gpointer data);
 void remmina_main_on_action_application_preferences(GSimpleAction *action, GVariant *param, gpointer data);
 void remmina_main_on_action_application_quit(GSimpleAction *action, GVariant *param, gpointer data);
 void remmina_main_on_action_connection_connect(GSimpleAction *action, GVariant *param, gpointer data);
@@ -127,5 +133,6 @@ void remmina_main_on_action_tools_export(GSimpleAction *action, GVariant *param,
 void remmina_main_on_action_tools_import(GSimpleAction *action, GVariant *param, gpointer data);
 void remmina_main_on_action_expand(GSimpleAction *action, GVariant *param, gpointer data);
 void remmina_main_on_action_collapse(GSimpleAction *action, GVariant *param, gpointer data);
+void remmina_main_on_action_search_toggle(GSimpleAction *action, GVariant *param, gpointer data);
 
 G_END_DECLS
