@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2010-2011 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2021 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2022 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -236,9 +236,16 @@ RemminaPluginService remmina_plugin_manager_service =
 	remmina_pref_get_sshtunnel_port,
 	remmina_pref_get_ssh_loglevel,
 	remmina_pref_get_ssh_parseconfig,
+	remmina_pref_keymap_get_table,
 	remmina_pref_keymap_get_keyval,
 
+	_remmina_info,
+	_remmina_message,
 	_remmina_debug,
+	_remmina_warning,
+	_remmina_audit,
+	_remmina_error,
+	_remmina_critical,
 	remmina_log_print,
 	remmina_log_printf,
 
@@ -349,7 +356,7 @@ void remmina_plugin_manager_init()
 	while(sple != NULL) {
 		sp = (RemminaSecretPlugin*)sple->data;
 		if (sp->init()) {
-			g_print("The %s secret plugin  has been initialized and it will be your default secret plugin\n",
+			g_print("The %s secret plugin has been initialized and it will be your default secret plugin\n",
 				sp->name);
 			remmina_secret_plugin = sp;
 			break;
