@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2009-2011 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2021 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2022 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,6 +37,7 @@
 #pragma once
 
 #include "config.h"
+#include <gtk/gtk.h>
 
 #define IDLE_ADD        gdk_threads_add_idle
 #define TIMEOUT_ADD     gdk_threads_add_timeout
@@ -87,14 +88,20 @@ void remmina_public_popup_position(GtkMenu *menu, gint *x, gint *y, gboolean *pu
 /* Combine two paths into one by correctly handling trailing slash. Return newly allocated string */
 gchar *remmina_public_combine_path(const gchar *path1, const gchar *path2);
 
+/**
+ * Return a file descriptor handle for a unix socket
+ * @return a file descriptor (socket) or -1
+ *
+ * */
+gint remmina_public_open_unix_sock(const char *unixsock);
+
 /* Parse a server entry with server name and port */
+void remmina_public_get_server_port_old(const gchar *server, gint defaultport, gchar **host, gint *port);
 void remmina_public_get_server_port(const gchar *server, gint defaultport, gchar **host, gint *port);
 
 /* X */
 gboolean remmina_public_get_xauth_cookie(const gchar *display, gchar **msg);
 gint remmina_public_open_xdisplay(const gchar *disp);
-guint remmina_public_get_current_workspace(GdkScreen *screen);
-guint remmina_public_get_window_workspace(GtkWindow *gtkwindow);
 
 /* Find hardware keycode for the requested keyval */
 guint16 remmina_public_get_keycode_for_keyval(GdkKeymap *keymap, guint keyval);
@@ -102,6 +109,8 @@ guint16 remmina_public_get_keycode_for_keyval(GdkKeymap *keymap, guint keyval);
 gboolean remmina_public_get_modifier_for_keycode(GdkKeymap *keymap, guint16 keycode);
 /* Load a GtkBuilder object from a filename */
 GtkBuilder *remmina_public_gtk_builder_new_from_file(gchar *filename);
+/* Load a GtkBuilder object from a resource */
+GtkBuilder *remmina_public_gtk_builder_new_from_resource(gchar *resource);
 /* Change parent container for a widget */
 void remmina_public_gtk_widget_reparent(GtkWidget *widget, GtkContainer *container);
 /* Used to send desktop notifications */
