@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2009-2011 Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2021 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2022 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -169,7 +169,7 @@ remmina_ssh_auth_interactive(RemminaSSH *ssh)
 	gint ret;
 	gint n;
 	gint i;
-	const gchar *name, *instruction;
+	const gchar *name, *instruction = NULL;
 	//gchar *prompt,*ptr;
 
 	ret = SSH_AUTH_ERROR;
@@ -578,7 +578,7 @@ remmina_ssh_auth(RemminaSSH *ssh, const gchar *password, RemminaProtocolWidget *
 {
 	TRACE_CALL(__func__);
 	gint method;
-	enum remmina_ssh_auth_result rv;
+	enum remmina_ssh_auth_result rv = REMMINA_SSH_AUTH_NULL;
 
 	/* Check known host again to ensure it’s still the original server when user forks
 	 * a new session from existing one */
@@ -870,11 +870,11 @@ remmina_ssh_auth_gui(RemminaSSH *ssh, RemminaProtocolWidget *gp, RemminaFile *re
 {
 	TRACE_CALL(__func__);
 	gchar *keyname;
-	gchar *pwdfkey;
+	gchar *pwdfkey = NULL;
 	gchar *message;
 	gchar *current_pwd;
 	gchar *current_user;
-	const gchar *instruction;
+	const gchar *instruction = NULL;
 	gint ret;
 	size_t len;
 	guchar *pubkey;
@@ -2315,6 +2315,7 @@ remmina_ssh_shell_new_from_file(RemminaFile *remminafile)
 	shell->master = -1;
 	shell->slave = -1;
 	shell->exec = g_strdup(remmina_file_get_string(remminafile, "exec"));
+	shell->run_line = g_strdup(remmina_file_get_string(remminafile, "run_line"));
 
 	return shell;
 }
@@ -2343,6 +2344,10 @@ remmina_ssh_call_exit_callback_on_main_thread(gpointer data)
 	RemminaSSHShell *shell = (RemminaSSHShell *)data;
 	if (shell->exit_callback)
 		shell->exit_callback(shell->user_data);
+	if (shell) {
+		remmina_ssh_shell_free(shell);
+		shell = NULL;
+	}
 	return FALSE;
 }
 
@@ -2362,10 +2367,10 @@ remmina_ssh_shell_thread(gpointer data)
 	gint buf_len;
 	gint len;
 	gint i, ret;
-	const gchar *filename;
+	gchar *filename;
 	const gchar *dir;
 	const gchar *sshlogname;
-	FILE *fp;
+	FILE *fp = NULL;
 
 	//gint screen;
 
@@ -2383,12 +2388,16 @@ remmina_ssh_shell_thread(gpointer data)
 
 	ssh_channel_request_pty(channel);
 
-	if (shell->exec && shell->exec[0])
+	if (shell->exec && shell->exec[0]) {
+		REMMINA_DEBUG ("Requesting an SSH exec channel");
 		ret = ssh_channel_request_exec(channel, shell->exec);
-	else
+	} else {
+		REMMINA_DEBUG ("Requesting an SSH shell channel");
 		ret = ssh_channel_request_shell(channel);
+	}
 	if (ret != SSH_OK) {
 		UNLOCK_SSH(shell)
+		REMMINA_WARNING ("Could not request shell");
 		// TRANSLATORS: The placeholder %s is an error message
 		remmina_ssh_set_error(REMMINA_SSH(shell), _("Could not request shell. %s"));
 		ssh_channel_close(channel);
@@ -2426,6 +2435,18 @@ remmina_ssh_shell_thread(gpointer data)
 		REMMINA_DEBUG("Saving session log to %s", filename);
 		fp = fopen(filename, "w");
 	}
+
+	g_free(filename);
+
+	REMMINA_DEBUG("Run_line: %s", shell->run_line);
+	if (!shell->closed && shell->run_line && shell->run_line[0]) {
+		LOCK_SSH(shell)
+		//TODO: Confirm assumption - assuming null terminated gchar string
+		ssh_channel_write(channel, shell->run_line, (gint)strlen(shell->run_line));
+		ssh_channel_write(channel, "\n", (gint)1); //TODO: Test this
+		UNLOCK_SSH(shell)
+		REMMINA_DEBUG("Run_line written to channel");
+	}
 	while (!shell->closed) {
 		timeout.tv_sec = 1;
 		timeout.tv_usec = 0;
@@ -2450,7 +2471,8 @@ remmina_ssh_shell_thread(gpointer data)
 			len = ssh_channel_poll(channel, i);
 			UNLOCK_SSH(shell)
 			if (len == SSH_ERROR || len == SSH_EOF) {
-				shell->closed = TRUE;
+				REMMINA_DEBUG("SSH shell error or EOF, closing the channel");
+				shell->closed=TRUE;
 				break;
 			}
 			if (len <= 0) continue;
@@ -2462,12 +2484,12 @@ remmina_ssh_shell_thread(gpointer data)
 			len = ssh_channel_read_nonblocking(channel, buf, len, i);
 			UNLOCK_SSH(shell)
 			if (len <= 0) {
-				shell->closed = TRUE;
+				shell->closed=TRUE;
 				break;
 			}
 			while (len > 0) {
 				ret = write(shell->slave, buf, len);
-				if (remmina_file_get_int (remminafile, "sshsavesession", FALSE)) {
+				if (fp != NULL) {
 					fwrite(buf, ret, 1, fp );
 					fflush(fp);
 				}
@@ -2479,7 +2501,7 @@ remmina_ssh_shell_thread(gpointer data)
 
 	LOCK_SSH(shell)
 	if (remmina_file_get_int (remminafile, "sshsavesession", FALSE))
-	fclose(fp);
+		fclose(fp);
 	shell->channel = NULL;
 	ssh_channel_close(channel);
 	ssh_channel_send_eof(channel);
@@ -2494,7 +2516,6 @@ remmina_ssh_shell_thread(gpointer data)
 	return NULL;
 }
 
-
 gboolean
 remmina_ssh_shell_open(RemminaSSHShell *shell, RemminaSSHExitFunc exit_callback, gpointer data)
 {
@@ -2556,6 +2577,10 @@ remmina_ssh_shell_free(RemminaSSHShell *shell)
 		g_free(shell->exec);
 		shell->exec = NULL;
 	}
+	if (shell->run_line) {
+		g_free(shell->run_line);
+		shell->run_line = NULL;
+	}
 	/* It’s not necessary to close shell->slave since the other end (vte) will close it */;
 	remmina_ssh_free(REMMINA_SSH(shell));
 }
