@@ -1,6 +1,6 @@
 /*
  * Remmina - The GTK+ Remote Desktop Client
- * Copyright (C) 2016-2021 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2022 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,6 +38,10 @@
  * @author Antenore Gatta and Giovanni Panozzo
  * @date 12 Feb 2018
  *
+ * Since October 29, 2021 data is not collected nor sent to remmina.org anymore. All the
+ * code intended to send data has been removed. The following documentation
+ * has to be kept for those with versions of Remmina older than 1.4.22.
+ *
  * When Remmina starts asks the user to share some usage statistics
  * with the Remmina developers. As per the opt-in model
  * (https://en.wikipedia.org/wiki/Opt-in_email), without the consent of the user,
@@ -110,8 +114,8 @@
  *    "ACTIVESECRETPLUGIN": {
  *        "plugin_name": "kwallet"
  *    }
- *    "HASMASTERPASSWORD": {
- *        "master_password_status": "OFF"
+ *    "HASPRIMARYPASSWORD": {
+ *        "primary_password_status": "OFF"
  *    }
  *
  * }
@@ -127,7 +131,7 @@
  *  - Protocols used
  *  - Last time each protocol has been used (globally).
  *
- * @see https://www.remmina.org/wp for more info.
+ * @see https://www.remmina.org for more info.
  */
 
 
@@ -168,48 +172,6 @@ struct ProfilesData {
 	gchar datestr;
 };
 
-JsonNode *remmina_stats_get_uid()
-{
-	TRACE_CALL(__func__);
-	JsonNode *r;
-	GChecksum *chs;
-	const gchar *uname, *hname;
-	const gchar *uid_suffix;
-	gchar *uid_prefix;
-	gchar *uid;
-
-	/** @warning this function is usually executed on a dedicated thread,
-	 * not on the main thread
-	 */
-
-	if (remmina_pref.periodic_usage_stats_uuid_prefix == NULL || remmina_pref.periodic_usage_stats_uuid_prefix[0] == 0) {
-		/* Generate a new UUID_PREFIX for this installation */
-		uid_prefix = remmina_gen_random_uuid();
-		if (remmina_pref.periodic_usage_stats_uuid_prefix)
-			g_free(remmina_pref.periodic_usage_stats_uuid_prefix);
-		remmina_pref.periodic_usage_stats_uuid_prefix = uid_prefix;
-		remmina_pref_save();
-	}
-
-	uname = g_get_user_name();
-	hname = g_get_host_name();
-	chs = g_checksum_new(G_CHECKSUM_SHA256);
-	g_checksum_update(chs, (const guchar*)uname, strlen(uname));
-	g_checksum_update(chs, (const guchar*)hname, strlen(hname));
-	uid_suffix = g_checksum_get_string(chs);
-
-	uid = g_strdup_printf("%s-%.10s", remmina_pref.periodic_usage_stats_uuid_prefix, uid_suffix);
-	g_checksum_free(chs);
-
-	r = json_node_alloc();
-	json_node_init_string(r, uid);
-
-	g_free(uid);
-
-	return r;
-
-}
-
 JsonNode *remmina_stats_get_os_info()
 {
 	TRACE_CALL(__func__);
@@ -777,12 +739,12 @@ JsonNode *remmina_stats_get_secret_plugin()
 }
 
 /**
- * Add a JSON member HASMASTERPASSWORD which shows the status of the master password.
+ * Add a JSON member HASPRIMARYPASSWORD which shows the status of the master password.
  *
- * @return a JSON Node structure containing the status of the master password
+ * @return a JSON Node structure containing the status of the primary password
  *
  */
-JsonNode *remmina_stats_get_master_password_status()
+JsonNode *remmina_stats_get_primary_password_status()
 {
 	TRACE_CALL(__func__);
 
@@ -792,8 +754,8 @@ JsonNode *remmina_stats_get_master_password_status()
 	b = json_builder_new();
 	json_builder_begin_object(b);
 
-	json_builder_set_member_name(b, "master_password_status");
-	if (remmina_pref_get_boolean("use_master_password")) {
+	json_builder_set_member_name(b, "primary_password_status");
+	if (remmina_pref_get_boolean("use_primary_password")) {
 		json_builder_add_string_value(b, "ON");
 	} else {
 		json_builder_add_string_value(b, "OFF");
@@ -809,7 +771,7 @@ JsonNode *remmina_stats_get_master_password_status()
 /**
  * Add a json member KIOSK which shows the status of the kiosk.
  *
- * @return a JSON Node structure containing the status of the master password
+ * @return a JSON Node structure containing the status of the primary password
  *
  */
 JsonNode *remmina_stats_get_kiosk_mode()
@@ -836,8 +798,6 @@ JsonNode *remmina_stats_get_kiosk_mode()
 	return r;
 }
 
-
-
 /**
  * Get all statistics in JSON format to send periodically to the PHP server.
  * The caller should free the returned buffer with g_free()
@@ -854,10 +814,6 @@ JsonNode *remmina_stats_get_all()
 	b = json_builder_new();
 	json_builder_begin_object(b);
 
-	n = remmina_stats_get_uid();
-	json_builder_set_member_name(b, "UID");
-	json_builder_add_value(b, n);
-
 	n = remmina_stats_get_version();
 	json_builder_set_member_name(b, "REMMINAVERSION");
 	json_builder_add_value(b, n);
@@ -898,8 +854,8 @@ JsonNode *remmina_stats_get_all()
 	json_builder_set_member_name(b, "ACTIVESECRETPLUGIN");
 	json_builder_add_value(b, n);
 
-	n = remmina_stats_get_master_password_status();
-	json_builder_set_member_name(b, "HASMASTERPASSWORD");
+	n = remmina_stats_get_primary_password_status();
+	json_builder_set_member_name(b, "HASPRIMARYPASSWORD");
 	json_builder_add_value(b, n);
 
 	n = remmina_stats_get_kiosk_mode();
