@@ -2,7 +2,7 @@
  * Remmina - The GTK+ Remote Desktop Client
  * Copyright (C) 2009 - Vic Lee
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2021 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2022 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -284,7 +284,7 @@ GtkDialog* remmina_string_list_new(gboolean two_columns, const gchar *fields_sep
 	string_list = g_new0(RemminaStringList, 1);
 	string_list->priv = g_new0(RemminaStringListPriv, 1);
 
-	string_list->builder = remmina_public_gtk_builder_new_from_file("remmina_string_list.glade");
+	string_list->builder = remmina_public_gtk_builder_new_from_resource("/org/remmina/Remmina/src/../data/ui/remmina_string_list.glade");
 	string_list->dialog = GTK_DIALOG(gtk_builder_get_object(string_list->builder, "DialogStringList"));
 
 	string_list->liststore_items = GTK_LIST_STORE(GET_OBJECT("liststore_items"));
